/* cybunit

	A collection of functions, taken from CybUnit.
	This was a former Pascal Unit made by the CyberHiPriest in 1997.
	Recoded in C/SDL/Linux by CHP in 2002.
*/

#include <stdlib.h>
#include "cybunit.h"

/*
 * Return the pixel value at (x, y)
 * NOTE: The surface must be locked before calling this!
 */
SDL_Color getpixel(SDL_Surface *surface, int x, int y)
{
      SDL_Color mycolor;
      Uint8 r,g,b;
      Uint32 pixel;
      int bpp = surface->format->BytesPerPixel;

      /* Here p is the address to the pixel we want to retrieve */
      Uint8 *p = (Uint8 *)surface->pixels + y * surface->pitch + x * bpp;

      switch(bpp) {
      case 1:
        pixel= (Uint32) (*p);
        break;
      case 2:
        pixel= (Uint32) (*(Uint16 *)p);
        break;
      case 3:
        if(SDL_BYTEORDER == SDL_BIG_ENDIAN)
            pixel= p[0] << 16 | p[1] << 8 | p[2];
        else
            pixel= p[0] | p[1] << 8 | p[2] << 16;
        break;
      case 4:
            pixel=*(Uint32 *)p;
      break;
      default:
            pixel=0;       /* shouldn't happen, but avoids warnings */
      }
      SDL_GetRGB(pixel,surface->format,&r,&g,&b);
      mycolor.r=r;
      mycolor.g=g;
      mycolor.b=b;
      return mycolor;
}

void putpixel(SDL_Surface *screen, int x, int y,
                                    Uint8 R, Uint8 G, Uint8 B)
{
      Uint32 color = SDL_MapRGB(screen->format, R, G, B);
      switch (screen->format->BytesPerPixel)
      {
            case 1: // Assuming 8-bpp
            {
                  Uint8 *bufp;
                  bufp = (Uint8 *)screen->pixels + y*screen->pitch + x;
                  *bufp = color;
            }
            break;
            case 2: // Probably 15-bpp or 16-bpp
            {
                  Uint16 *bufp;
                  bufp = (Uint16 *)screen->pixels + y*screen->pitch/2 + x;
                  *bufp = color;
            }
            break;
            case 3: // Slow 24-bpp mode, usually not used
            {
                  Uint8 *bufp;
                  bufp = (Uint8 *)screen->pixels + y*screen->pitch + x * 3;
                  if(SDL_BYTEORDER == SDL_LIL_ENDIAN)
                  {
                        bufp[0] = color;
                        bufp[1] = color >> 8;
                        bufp[2] = color >> 16;
                  } else {
                        bufp[2] = color;
                        bufp[1] = color >> 8;
                        bufp[0] = color >> 16;
                  }
            }
            break;
            case 4: // Probably 32-bpp
            {
                  Uint32 *bufp;
                  bufp = (Uint32 *)screen->pixels + y*screen->pitch/4 + x;
                  *bufp = color;
            }
            break;
      }
}


void Slock(SDL_Surface *screen)
{
      if ( SDL_MUSTLOCK(screen) )
            SDL_LockSurface(screen);
}


void Sulock(SDL_Surface *screen)
{
      if ( SDL_MUSTLOCK(screen) )
            SDL_UnlockSurface(screen);
}

SDL_Surface* MakeWindow(unsigned int x, unsigned y, unsigned char bbp)
{
      SDL_Surface *screen;
      
      if ( SDL_Init(SDL_INIT_VIDEO) < 0 )
      {
            printf("Unable to init SDL Video library: %s\n", SDL_GetError());
            exit(1);
      }

/*
      screen=SDL_SetVideoMode(x,y,bbp,SDL_HWSURFACE|SDL_DOUBLEBUF|SDL_HWACCEL);
*/
      screen=SDL_SetVideoMode(x,y,bbp,SDL_SWSURFACE|SDL_DOUBLEBUF);

      if ( screen == NULL )
      {
            printf("Unable to set %dx%d video: %s\n",x,y, SDL_GetError());
            exit(1);
      }
      return screen;
}

void ClsScreen(SDL_Surface *screen)
{
      SDL_Rect myRect;
      myRect.x=0;
      myRect.y=0;
      myRect.w=screen->w;
      myRect.h=screen->h;
      SDL_FillRect(screen, &myRect, 0);
}

void Hline(SDL_Surface* screen, unsigned int y, Uint8 R, Uint8 G, Uint8 B)
{
      int i;
      for(i=0;i<screen->w;i++)
            putpixel(screen,i,y,R,G,B);
}

void Vline(SDL_Surface* screen, unsigned int x, Uint8 R, Uint8 G, Uint8 B)
{
      int i;
      for(i=0;i<screen->h;i++)
            putpixel(screen,x,i,R,G,B);
}

void DrawLine(SDL_Surface* screen,
                  unsigned int x1,
                  unsigned int x2,
                  unsigned int y1,
                  unsigned int y2,
                  Uint8 R, Uint8 G, Uint8 B)
{
      int dx,dy,inc1,inc2,d,x,y,xend,yend,yinc,xinc;

      /* check if the line is just a small vertical line */
      if(x1==x2){
            if(y1 > y2) { yend = y1; y1 = y2; y2=yend;}
            for(;y1<=y2;y1++) putpixel(screen,x1,y1,R,G,B);
      }

      else {
            /* check if the line is just a small horizontal line */
            if (y2==y1) {
                  if(x1 > x2){ xend = x1; x1=x2; x2=xend;}
                  for(;x1<=x2;x1++) putpixel(screen,x1,y1,R,G,B);
            }     
            else {
                  /* The line is a real line.. Start drawing it using
                  Bresenham's Algorithm. */
                  dx= abs(x2-x1); dy=abs(y2-y1);
                  /* Determine whether the line should be drawn along
                  the x or y axis. */
                  if(dx>=dy) {
                        /* slope < 1, so draw along the x - axis */
                        if(x1>x2) {
                              x = x2; y = y2; xend = x1;
                              if(dy==0) yinc=0;
                              else if (y2>y1) yinc=-1; else yinc=1;
                        }
                        else {
                              x = x1; y= y1; xend = x2;
                              if(dy==0) yinc=0;
                              else if(y2>y1) yinc=1; else yinc=-1;
                        }
                        /* Draw first point */
                        inc1=2*dy; d= inc1-dx; inc2=2*(dy-dx);
                        putpixel(screen,x,y,R,G,B);
                        /* Draw the rest of the points */
                        while(x<xend){
                              x++;
                              if(d<0) d+= inc1;
                              else { y += yinc; d += inc2; }
                              putpixel(screen,x,y,R,G,B);
                        }
                  }
                  else {
                        /* Draw along y - axis */
                        if(y1>y2) {
                              x=x2;y=y2;yend=y1;
                              if(dx==0) xinc=0;
                              else if(x2>x1) xinc= -1; else xinc=1;
                        }
                        else {
                              x=x1;y=y1;yend=y2;
                              if(dx==0)xinc=0;
                              else if(x2>x1) xinc=1; else xinc=-1;
                        }
                        /* first pixel */
                        inc1=2*dx; d=inc1-dy;inc2=2*(dx-dy);
                        putpixel(screen,x,y,R,G,B);
                        /* draw rest of pixels */
                        while(y<yend) {
                              y++;
                              if(d<0) d+=inc1;
                              else x+=xinc; d+=inc2;
                              putpixel(screen,x,y,R,G,B);
                        }
                  }
            }
      }
}

