#include <stdio.h>
#include <stdlib.h>
#include <math.h>
#include "cybunit.h"
#include <SDL/SDL.h>

#include <mm3dnow.h>

#define ITERATIONS 100
#define RESX 600
#define RESY 400

typedef struct
{
    float re;
    float im;
} cx ;

#define M64_AT_ADDRESS_OF(a)  ( *( (__m64 *)(&(a)) ) )

#warning Only compile with -O1 and not higher. (intrinsics are screwed otherwise)

float fTwo[2]={2,2};

/* this routine will calculate 1 frame (consisting of RESX, RESY points) of calculated mandelbrot iterations.
*/
void DrawMandelFrame(SDL_Surface* screen, cx start, cx end)
{
    cx step,current,square;
	float simd[8];
	register __m64 acc, temp, squared, recipsquareroot, two;
    int x,y,iteration,i;
    Uint32 trigger[2];
	
	iteration = 0;
    current   = start;
	
    /* calculate the step of advancing over the complex plane. */
    step.re = (end.re - start.re)/RESX;
    step.im = (end.im - start.im)/RESY;

    Uint32 iTime = SDL_GetTicks();

	
    for(y=0;y<RESY;y++)
    {
        for(x=0;x<RESX;x++)
        {
            square=current;
	        for(i=0;i<ITERATIONS;i++)
            {			
				// sort the variables for quick SIMD operations
				simd[5] = simd[1] = simd[2] = simd[0] = square.re;
				simd[3] = simd[4] = simd[6] = simd[7] = square.im;
				simd[5] = -simd[5];
	
				__asm__ __volatile__ ("femms\n\t");
				two = M64_AT_ADDRESS_OF(fTwo);
	
				// do the first calculation
				temp = _m_pfadd( _m_pfsub ( _m_pfmul (M64_AT_ADDRESS_OF(simd[0]),M64_AT_ADDRESS_OF(simd[2])),
		  							        _m_pfmul (M64_AT_ADDRESS_OF(simd[4]),M64_AT_ADDRESS_OF(simd[6]))),
							     M64_AT_ADDRESS_OF(current));
	
				// remember the temp value in square for next iteration.
				M64_AT_ADDRESS_OF(square) = temp;
				
				// do the second part of the calculation (FAST low precision version)
			    squared = _m_pfmul (temp , temp);
				acc     = _m_pfacc ( squared, squared );
				recipsquareroot = _m_pfrsqrt(acc);
				temp = _m_pfmul (acc, recipsquareroot);
				M64_AT_ADDRESS_OF(trigger) = _m_pfcmpgt (temp,two);
				__asm__ __volatile__ ("femms\n\t");
				
				if(trigger[1])
                {
				    putpixel(screen,x,y,0,i<<3,i<<1);
                    break;
                }
            }
            iteration++;
            current.re+=step.re;
        }
        current.re=start.re;
        current.im+=step.im;
    }
	
    Uint32 iTimePassed = SDL_GetTicks() - iTime;
    printf("One frame of %d pixels took %d milliseconds.\n", RESX*RESY, iTimePassed);
    SDL_Flip(screen);
}
    
int main (void)
{
    cx start,end;  
    SDL_Surface* screen;
    SDL_Event event;
    
    screen=MakeWindow(RESX,RESY,0);
    SDL_WM_SetCaption(" ~ MandelBrot Zoomer (C) 2003 Freedom Systems ~ "," Mandel ");

    start.re=-2.25;
    start.im=-1;
    end.re=0.75;
    end.im=1;
    
    DrawMandelFrame(screen,start,end);

    while ( 1 )
    {
      if(SDL_PollEvent(&event) >= 0)
           switch (event.type)
           {
           case SDL_QUIT:
           {
               printf("You closed the main window, quitting and flushing SDL buffers.\n");
               SDL_Quit();
               return 0;
           }
           break;
	   case SDL_KEYUP:
	   {
		printf("You closed the main window, quitting and flushing SDL buffers.\n");
               SDL_Quit();
               return 0;
	   }
	   break;
	   }
    SDL_Delay(50);
    }  

    return 0;
}

