#include <stdio.h>
#include <stdlib.h>
#include <math.h>
#include "cybunit.h"
#include <SDL/SDL.h>
#include <mm3dnow.h>

#define ITERATIONS 500
#define RESX 600
#define RESY 400

typedef struct
{
    float re;
    float im;
} cx ;

float fTwo[2]={2,2};

/* this routine will calculate 1 frame (consisting of RESX, RESY points) of calculated mandelbrot iterations.
*/
void DrawMandelFrame(SDL_Surface* screen, cx start, cx end)
{
    cx step,current,square;
	float simd[8];
    int x,y,iteration,i;
    Uint32 trigger[2];
	
	iteration = 0;
    current   = start;
	
    /* calculate the step of advancing over the complex plane. */
    step.re = (end.re - start.re)/RESX;
    step.im = (end.im - start.im)/RESY;

    Uint32 iTime = SDL_GetTicks();

	
    for(y=0;y<RESY;y++)
    {
        for(x=0;x<RESX;x++)
        {
            square=current;
            for(i=0;i<ITERATIONS;i++)
            {
				// sort the variables for quick SIMD operations
				simd[5] = simd[1] = simd[2] = simd[0] = square.re;
				simd[3] = simd[4] = simd[6] = simd[7] = square.im;
				simd[5] = -simd[5];

				// inline 3Dnow assembly follows 
				// %2 = &simd[0]
				// %3 = &simd[2]
				// %4 = &simd[4]
				// %5 = &simd[6]
				// %6 = &current
				// %0 = &square
				// %7 = &fTwo
				// %1 = &trigger
				__asm__ __volatile__ (	"femms 					\n\t"
										"movq %2,%%mm0          \n\t"						
										"pfmul %3,%%mm0         \n\t"						
										"movq %4,%%mm1          \n\t"						
										"pfmul %5,%%mm1         \n\t"						
										"pfsub %%mm1,%%mm0      \n\t"						
										"pfadd %6,%%mm0         \n\t"						
										"movq %%mm0,%0          \n\t"						
										"pfmul %%mm0,%%mm0      \n\t"						
										"pfacc %%mm0,%%mm0      \n\t"						
										"pfrsqrt %%mm0,%%mm1    \n\t"						
										"pfmul %%mm1,%%mm0      \n\t"						
										"movq %7,%%mm0          \n\t"						
										"pfcmpgt %%mm1,%%mm0    \n\t"						
										"movq %%mm0,%1          \n\t"						
										"femms                      "						
										: "=m" (square), "=m" (trigger)
										: "m" (simd[0]), "m" (simd[2]), "m" (simd[4]), "m" (simd[6]), "m" (current), "m" (fTwo));
				
				if(trigger[1])
                {
				    putpixel(screen,x,y,0,i<<3,i<<1);
                    break;
                }
            }
            iteration++;
            current.re+=step.re;
        }
        current.re=start.re;
        current.im+=step.im;
    }
	
    Uint32 iTimePassed = SDL_GetTicks() - iTime;
    printf("One frame of %d pixels took %d milliseconds.\n", RESX*RESY, iTimePassed);
    SDL_Flip(screen);
}
    
int main (void)
{
    cx start,end;  
    SDL_Surface* screen;
    SDL_Event event;
    
    screen=MakeWindow(RESX,RESY,0);
    SDL_WM_SetCaption(" ~ MandelBrot Zoomer (C) 2003 Freedom Systems ~ "," Mandel ");

    start.re=-2.25;
    start.im=-1;
    end.re=0.75;
    end.im=1;
    
    DrawMandelFrame(screen,start,end);

    while ( 1 )
    {
      if(SDL_PollEvent(&event) >= 0)
           switch (event.type)
           {
           case SDL_QUIT:
           {
               printf("You closed the main window, quitting and flushing SDL buffers.\n");
               SDL_Quit();
               return 0;
           }
           break;
	   case SDL_KEYUP:
	   {
		printf("You closed the main window, quitting and flushing SDL buffers.\n");
               SDL_Quit();
               return 0;
	   }
	   break;
	   }
    SDL_Delay(50);
    }  

    return 0;
}

