#include <stdio.h>
#include <stdlib.h>
#include <math.h>
#include "cybunit.h"
#include <SDL/SDL.h>

#define ITERATIONS 100
#define RESX 600
#define RESY 400

typedef struct
{
    float re;
    float im;
} cx ;

/* this routine will calculate 1 frame (consisting of RESX, RESY points) of calculated mandelbrot iterations.
*/
void DrawMandelFrame(SDL_Surface* screen, cx start, cx end)
{
    cx step;
    cx current,square;
    int x,y,iteration,i;
    float myabs;
	cx saved;
    iteration = 0;
    current   = start;
    
    /* calculate the step of advancing over the complex plane. */
    step.re = (end.re - start.re)/RESX;
    step.im = (end.im - start.im)/RESY;

    Uint32 iTime = SDL_GetTicks();

    for(y=0;y<RESY;y++)
    {
        for(x=0;x<RESX;x++)
        {
            square=current;
            for(i=0;i<ITERATIONS;i++)
            {
                saved = square;
				// first part
                square.re = (saved.re * saved.re) - (saved.im * saved.im) + current.re;
                square.im = (saved.re * saved.im) + (saved.re * saved.im) + current.im;
				// second part
				myabs = sqrt( (square.re * square.re) + (square.im * square.im));
                if(myabs>2.0)
                {
				    putpixel(screen,x,y,0,i<<3,i<<1);
                    break;
                }
            }
            iteration++;
            current.re+=step.re;
        }
        current.re=start.re;
        current.im+=step.im;
    }

    Uint32 iTimePassed = SDL_GetTicks() - iTime;
    printf("One frame of %d pixels took %d milliseconds.\n", RESX*RESY, iTimePassed);
    SDL_Flip(screen);
}
    
int main (void)
{
    cx start,end;  
    SDL_Surface* screen;
    SDL_Event event;
    
    screen=MakeWindow(RESX,RESY,0);
    SDL_WM_SetCaption(" ~ MandelBrot Zoomer (C) 2003 Freedom Systems ~ "," Mandel ");

    start.re=-2.25;
    start.im=-1;
    end.re=0.75;
    end.im=1;
    
    DrawMandelFrame(screen,start,end);

    while ( 1 )
    {
      if(SDL_PollEvent(&event) >= 0)
           switch (event.type)
           {
           case SDL_QUIT:
           {
               printf("You closed the main window, quitting and flushing SDL buffers.\n");
               SDL_Quit();
               return 0;
           }
           break;
	   case SDL_KEYUP:
	   {
		printf("You closed the main window, quitting and flushing SDL buffers.\n");
               SDL_Quit();
               return 0;
	   }
	   break;
	   }
    SDL_Delay(50);
    }  

    return 0;
}

